from df_notifications.channels import BaseChannel


class TestChannel(BaseChannel):
    key = "test"
    template_parts = ["msg"]
    title_part = "msg"
    additional_data = []
    config_items = []

    def send(self, users, context):
        print(context)



# Generated by Django 4.1.5 on 2023-01-16 09:31

import df_notifications.fields
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserDevice',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=255, null=True, verbose_name='Name')),
                ('active', models.BooleanField(default=True, help_text='Inactive devices will not be sent notifications', verbose_name='Is active')),
                ('date_created', models.DateTimeField(auto_now_add=True, null=True, verbose_name='Creation date')),
                ('device_id', models.CharField(blank=True, db_index=True, help_text='Unique device identifier', max_length=255, null=True, verbose_name='Device ID')),
                ('registration_id', models.TextField(verbose_name='Registration token')),
                ('type', models.CharField(choices=[('ios', 'ios'), ('android', 'android'), ('web', 'web')], max_length=10)),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_query_name='fcmdevice', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'User device',
                'verbose_name_plural': 'User devices',
            },
        ),
        migrations.CreateModel(
            name='NotificationHistory',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('channel', df_notifications.fields.NoMigrationsChoicesField(max_length=255)),
                ('template_prefix', models.CharField(max_length=255)),
                ('content', models.JSONField(blank=True, default=dict)),
                ('instance_id', models.PositiveBigIntegerField(blank=True, null=True)),
                ('content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype')),
                ('users', models.ManyToManyField(blank=True, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name_plural': 'Notification history',
            },
        ),
        migrations.AddIndex(
            model_name='notificationhistory',
            index=models.Index(fields=['template_prefix', 'created'], name='df_notifica_templat_fc63aa_idx'),
        ),
        migrations.AddIndex(
            model_name='notificationhistory',
            index=models.Index(fields=['channel', 'created'], name='df_notifica_channel_ecbe66_idx'),
        ),
        migrations.AddIndex(
            model_name='notificationhistory',
            index=models.Index(fields=['content_type', 'instance_id', 'created'], name='df_notifica_content_1618f4_idx'),
        ),
    ]



from df_notifications.decorators import register_reminder_model
from df_notifications.decorators import register_rule_model
from df_notifications.models import NotifiableModelMixin
from df_notifications.models import NotificationModelReminder
from df_notifications.models import NotificationModelRule
from django.contrib.auth.models import User
from django.db import models
from django.db.models import Q
from django.db.models import QuerySet
from typing import List
from typing import Optional

import json


class Post(NotifiableModelMixin):
    title = models.CharField(max_length=255)
    description = models.TextField()
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)
    author = models.ForeignKey(User, on_delete=models.CASCADE)
    is_published = models.BooleanField(default=False)

    def json_data(self):
        return json.dumps(
            {
                "title": self.title,
                "description": self.description,
                "is_published": self.is_published,
            }
        )


@register_rule_model
class PostNotificationRule(NotificationModelRule):
    model = Post
    admin_list_display = [
        "template",
        "channel",
        "is_published_prev",
        "is_published_next",
    ]
    tracking_fields = ["is_published"]

    is_published_prev = models.BooleanField(default=False, null=True)
    is_published_next = models.BooleanField(default=True)

    def get_users(self, instance: Post) -> List[User]:
        return [instance.author]

    @classmethod
    def get_queryset(
        cls, instance: Post, prev: Optional[Post]
    ) -> QuerySet["PostNotificationRule"]:
        qs = cls.objects.filter(
            is_published_next=instance.is_published,
        )

        if prev:
            qs = qs.filter(
                Q(is_published_prev__isnull=True)
                | Q(is_published_prev=prev.is_published),
            )

        return qs


@register_reminder_model
class PostNotificationReminder(NotificationModelReminder):
    MODIFIED_MODEL_FIELD = "updated"
    model = Post
    admin_list_display = [
        "template",
        "channel",
        "delay",
        "cooldown",
        "repeat",
        "is_published",
        "action",
    ]

    is_published = models.BooleanField(default=True)

    def get_users(self, instance: Post) -> List[User]:
        return [instance.author]



from celery import current_app as app
from df_notifications.models import BaseModelReminder
from df_notifications.models import NotificationModelMixin
from df_notifications.models import User
from df_notifications.utils import send_notification
from django.apps import apps
from django.conf import settings
from typing import Type


@app.on_after_finalize.connect
def setup_periodic_tasks(sender, **kwargs):
    sender.add_periodic_task(
        settings.DF_NOTIFICATIONS["REMINDERS_CHECK_PERIOD"], register_reminders.s()
    )


@app.task()
def register_reminders():
    for model in apps.get_models():
        if issubclass(model, BaseModelReminder):
            model.invoke()


@app.task
def send_model_notification_async(model_notification_class, notification_pk, model_pk):
    ModelNotification: Type[NotificationModelMixin] = apps.get_model(
        model_notification_class
    )
    notification = ModelNotification.objects.get(pk=notification_pk)
    instance = ModelNotification.model.objects.get(pk=model_pk)
    NotificationModelMixin.send(notification, instance)


@app.task
def send_notification_async(user_ids, channel_name, template_prefixes, context):
    users = User.objects.filter(id__in=user_ids)
    send_notification(users, channel_name, template_prefixes, context)



from df_notifications.channels import BaseChannel
from df_notifications.channels import User
from df_notifications.models import NotificationHistory
from django.conf import settings
from django.template.loader import render_to_string
from django.utils.module_loading import import_string
from functools import cache
from typing import Any
from typing import Dict
from typing import List
from typing import Union


@cache
def get_channel_instance(channel) -> BaseChannel:
    return import_string(settings.DF_NOTIFICATIONS["CHANNELS"][channel])()


def send_notification(
    users: List[User],
    channel: str,
    template_prefixes: Union[List[str], str],
    context: Dict[str, Any],
):
    if isinstance(template_prefixes, str):
        template_prefixes = [template_prefixes]

    channel_instance = get_channel_instance(channel)
    parts = {}
    for part in channel_instance.template_parts:
        templates = []
        for prefix in template_prefixes:
            templates.append(f"{prefix}{channel}__{part}")
            templates.append(f"{prefix}{part}")
        parts[part] = render_to_string(templates, context=context)

    channel_instance.send(users, {**context, **parts})

    notification = NotificationHistory.objects.create(
        channel=channel,
        template_prefix=template_prefixes[0],
        content=parts if settings.DF_NOTIFICATIONS["SAVE_HISTORY_CONTENT"] else "",
        instance=context.get("instance"),
    )
    notification.users.set(users)
    return notification



# Generated by Django 4.1.5 on 2023-01-26 10:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('df_notifications', '0001_initial'),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='notificationhistory',
            name='df_notifica_templat_fc63aa_idx',
        ),
        migrations.RenameField(
            model_name='notificationhistory',
            old_name='template_prefix',
            new_name='template',
        ),
        migrations.AddIndex(
            model_name='notificationhistory',
            index=models.Index(fields=['template', 'created'], name='df_notifica_templat_ec4b48_idx'),
        ),
    ]



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class DFNotificationsConfig(AppConfig):
    name = "df_notifications"
    verbose_name = _("DjangoFlow Notifications")

    def ready(self):
        try:
            import df_notifications.signals  # noqa F401
        except ImportError:
            pass

    class Djangoflow:
        required_apps = [
            "df_notifications",
            "fcm_django",
            "django_slack",
        ]
        api_path = "notifications/"



# Generated by Django 4.1.5 on 2023-01-17 09:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('df_notifications', '0001_initial'),
        ('test_app', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='postnotificationreminder',
            name='history',
            field=models.ManyToManyField(blank=True, editable=False, to='df_notifications.notificationhistory'),
        ),
        migrations.AlterField(
            model_name='postnotificationrule',
            name='history',
            field=models.ManyToManyField(blank=True, editable=False, to='df_notifications.notificationhistory'),
        ),
    ]
